#############################################################################################################
### Analysis of the Tandmobiel dataset using the flexible frailty model as described in 				#
### SM14-AK040 "Semiparametric frailty model for clustered interval-censored data"					#
### A. Cetinyurek Yavuz & P. Lambert (2015)											#
#############################################################################################################


### SECTION 0: Setting the environment and extract the dataset ######################
#####################################################################################


# Set working environment
# ----------------------------------------------------------------------
setwd("C:/Users/AysunC/Documents/MCMC/Frailty")

# Load required R packages and functions
# ----------------------------------------------------------------------
source("bases.r");source("bsplbase.r");source("aPoisson.r")
library(lattice); library(splines); library(coda); library(numDeriv)
library(HI); library(ars); library(survival); library(MASS);library(mixtools);
library(LearnBayes); library(interval);library(bayesSurv);library(Icens)

# function for the identifiability constraint on the spline coefficients used for the baseline density specification
# ----------------------------------------------------------------------
standbase=function(thetax){
etax= as.vector(lB%*%thetax); 
thetax.cons=log(sum(exp(etax)))
thetax=thetax-thetax.cons
return(thetax) }

# function for the identifiability constraint on the spline coefficients used for the frailty density
# ----------------------------------------------------------------------
standfr=function(thetafrx){
etax= as.vector(lBfr%*%thetafrx); 
thetafrx.cons=log(sum(exp(etax)))
thetafrx=thetafrx-thetafrx.cons
return(thetafrx) }

### Section 0.1: Load the tandmobiel dataset  
# ----------------------------------------------------------------------
data(tandmob2) 
attach(tandmob2)


### Section 0.2: Specify teeth we want to analyze  
# (dteeth are corresponding deciduous teeth)
# ----------------------------------------------------------------------
pteeth <- c(14, 24, 34, 44, 15, 25, 35, 45) 
dteeth <- pteeth + 40   		# predecessors of ptheeth
nteeth <- length(pteeth) 


### Section 0.4: Dummies for horizontally symmetric teeth  
# Dummies for horizontally symmetric teeth (upper4 = teeth 14 and 24, lower4 = teeth 34 and 44, upper5 = teeth 15 and 25, lower5 = teeth 35 and 45) further used in the analysis:
# ----------------------------------------------------------------------------------------------------------
upper4 <- c(1, 1, 0, 0, 0, 0, 0, 0) 	# max4
lower4 <- c(0, 0, 1, 1, 0, 0, 0, 0) 	# man4
upper5 <- c(0, 0, 0, 0, 1, 1, 0, 0) 	# max5
lower5 <- c(0, 0, 0, 0, 0, 0, 1, 1) 	# man5



### Section 0.5 Variables  
# Variables from the whole dataset that are needed or otherwise interesting (timevars are variables that
# determine the interval where the emergence was recorded, dmfvars give the caries status of corresponding deciduous teeth):
# -------------------------------------------------------------------------------------------------------------
childvars <- c("IDNR", "GENDER", "GENDERNum", "DOB", "PROVINCE", "EDUC") 
timevars <- paste(c("EBEG.", "EEND."), rep(pteeth, rep(2, nteeth)), sep="") 
dmfvars <- paste("T", dteeth, ".DMF", sep="") 
vars <- c(childvars, timevars, dmfvars) 
print(vars) 


### Section 0.6: extract only selected variables from dataset  
# Take only interesting variables from the data and change some names. To see explanation of the variables, check the help page of tandmob2.
# ----------------------------------------------------------------------
tandmob2 <- tandmob2[, vars] 


### Section 0.7: change the name of Gender  
# Change the name of GENDERNum variable: 1 for GIRL, 0 for BOY
# ----------------------------------------------------------------------

names(tandmob2)[names(tandmob2) == "GENDERNum"] <- "GIRL" 
childvars[childvars == "GENDERNum"] <- "GIRL" 
vars[vars == "GENDERNum"] <- "GIRL" 


### Section 0.8: Take only children for who deciduous dmf score is available for all studied teeth: 
# -------------------------------------------------------------------------------------------------------
is.dmf <- !is.na(tandmob2$T54.DMF) & !is.na(tandmob2$T64.DMF) &
+ !is.na(tandmob2$T74.DMF) & !is.na(tandmob2$T84.DMF) & !is.na(tandmob2$T55.DMF) &
+ !is.na(tandmob2$T65.DMF) & !is.na(tandmob2$T75.DMF) & !is.na(tandmob2$T85.DMF)
tandmob2 <- tandmob2[is.dmf, ]


### Section 0.9: subtract 5.0 from time variables (i.e. time 0 in the analyzes = 5 years of age
# ---------------------------------------------------------------------------------------------
startage <- 5.0 
tandmob2[, timevars] <- tandmob2[, timevars] - startage 


### Section 0.10: Selection of 300 random subjects 
# ----------------------------------------------------------------------
# We divide the dataset for girls and boys
# tandmobgirl<- subset(tandmob2, GENDER=="girl" ) 
# tandmobboy<- subset(tandmob2, GENDER=="boy" ) 

# We randomly select 150 girls and 150 boys
# samplegirl <- tandmobgirl[sample(1:nrow(tandmobgirl), 150, replace=FALSE),] 
# sampleboy <- tandmobboy[sample(1:nrow(tandmobboy), 150, replace=FALSE),] 

# combine the two datasets
# tandmob2s<-rbind(samplegirl,sampleboy)
# IDNR<-tandmob2s$IDNR
# Write the ID of the selected sample of children to the ID300.dat document to be able to use the same sample
# write.csv(IDNR,file="C:/Users/Aysun/Documents/Frailty Paper/ID300.dat") 

# Once the selection of the sample is done and saved in Frailty Paper folder 
# to be able to use the same sample every time we use the data set
docdir <- "C:/Users/AysunC/Documents/Frailty Paper/"
ids <- read.table(paste(docdir, "ID300", ".dat",sep = ""), header = TRUE,sep=",")[, 2]
wanna <- match(tandmob2$IDNR, ids, nomatch = 0)
wanna <- as.logical(wanna)
wanna[wanna > 1] <- TRUE
tandmobl<- tandmob2[wanna, ]




### Section 0.11: reformat the data 	
## Now we have one row per child in the data.frame tandmob2s. We reformat the data such that there is one row per tooth 
## and create factors to determine the tooth
# ----------------------------------------------------------------------
nchild <- dim(tandmobl)[1] 
longdata <- list() 
for (i in 1:length(childvars)){ longdata[[i]] <- rep(tandmobl[[childvars[i]]], nteeth) } 
names(longdata) <- childvars 
longdata <- as.data.frame(longdata) 
longdata$TOOTH <- as.factor(rep(pteeth, rep(nchild, nteeth))) 
longdata$UPPER4 <- rep(upper4, rep(nchild, nteeth)) 
longdata$LOWER4 <- rep(lower4, rep(nchild, nteeth)) 
longdata$UPPER5 <- rep(upper5, rep(nchild, nteeth)) 
longdata$LOWER5 <- rep(lower5, rep(nchild, nteeth)) 
dmf <- numeric(); for (i in 1:nteeth) 
dmf <- c(dmf, tandmobl[[dmfvars[i]]]) 
ebeg <- numeric(); 
for (i in 1:nteeth) ebeg <- c(ebeg, tandmobl[[paste("EBEG.", pteeth[i], sep="")]]) 
eend <- numeric(); 
for (i in 1:nteeth) eend <- c(eend, tandmobl[[paste("EEND.", pteeth[i], sep="")]]) 
longdata$DMF <- dmf 
longdata$EBEG <- ebeg 
longdata$EEND <- eend 

rm(list=c("dmf", "ebeg", "eend")) 
longdata <- longdata[order(longdata$IDNR), ] 


### Section 0.12: Print some part of the data 
# ----------------------------------------------------------------------
print(longdata[1:16,]) 



### SECTION 1: Prepare the dataset for analysis using flexible frailty model ######################
###################################################################################################

# number of clusters and observations per cluster 
n=nrow(longdata);
ncl=nchild
ni=rep(n/ncl,ncl)

# Interval-censored data (yl,yr)
yl=longdata$EBEG
yr=longdata$EEND
d=rep(1,n)
d[is.na(yr)]=0
ksi0=1-(sum(is.na(yr))/nrow(longdata))
id=longdata$IDNR

up=8 				# Latest time for emergence,  because most of the premolars should have emerged at the age of 13 years (8+5). 
yr[is.na(yr)]=up		# When an observation is right censored, then yr is NA. We replace yr with the latest time for emergence
yl[is.na(yl)]=0
w.int<-(yr-yl)

### Define t(cens): the end of the study 
tcens=rep(7.5,n)			# We set the end of the observation period to 7.5 for all children, because maximum observed age was 7.4. 
sbin=seq(0,tcens[1],by=1/10)  # small bins u_i for the flexible density estimation
x=seq(0,1,length=up)

### Covariates	
# --------------------
sex=longdata$GIRL
dmf=longdata$DMF
dmf[is.na(dmf)]=1

man4=longdata$LOWER4
max5=longdata$UPPER5 
man5=longdata$LOWER5
		
sexdmf=sex*dmf		
sexman4=sex*man4
sexmax5=sex*max5
sexman5=sex*man5
dmfman4=dmf*man4
dmfmax5=dmf*max5
dmfman5=dmf*man5
triman4=sexdmf*man4
trimax5=sexdmf*max5
triman5=sexdmf*man5

### Model 5: including dmf, sex, and 3 dummies for horizontal symmetry and their 2- and 3-way interactions 	
# ----------------------------------------------------------------------------------------------------------
Z=cbind(dmf,sex,sexdmf,man4,max5,man5,sexman4,sexmax5,sexman5,dmfman4,dmfmax5,dmfman5,triman4,trimax5,triman5)
nZ=ncol(Z)

### Model 1: only using dmf (Uncomment next two lines for Model 1 estimates 
# Z=cbind(dmf)  
# nZ=ncol(Z)


### Midpoint Imputation 
# ------------------------
times=numeric(length(yr))
times[which(yr<=tcens)]=(0.5*yr[which(yr<=tcens)]+0.5*yl[which(yr<=tcens)])
times[which(yr>tcens)]=yl[which(yr>tcens)]	



### Naive Frequentist Cox PH model using midpoint imputation
# ----------------------------------------------------------------------
cox.model=coxph(Surv(times,d)~(Z))  
coeff.cox=unlist(cox.model$coef) ;  
summary(cox.model)


### Naive Cox Ph with frailty using midpoint imputation 
# ---------------------------------------------------------------------------
frt.m=coxph(Surv(times,d)~(Z+frailty(id,dist="gaussian")))
coeff.fr=frt.m$coef ; 					
summary(frt.m)


### Estimated Frailty terms from frailty model using midpoint imputation 
# --------------------------------------------------------------------------------------
frmin=min(round(exp(frt.m$frail),0));
frmax=max(round(exp(frt.m$frail),0))
x11();hist(frt.m$frail,freq=FALSE,breaks=seq(-6,6,by=.5),col='lavender')


### Initial values of z_g: Standardize the frailty terms	
# ---------------------------------------------------------------
frterm=frt.m$frail
frm=mean(frterm)
frsd=sd(frterm)
fr.s=(frterm-frm)/frsd


### The grid for the frailty terms	
# ----------------------------------------------
pseu.ctfr=hist(fr.s,freq=TRUE,breaks=seq(-6.,6.,by=.1))$counts 
uifr=hist(fr.s,freq=TRUE,breaks=seq(-6.,6.,by=0.1))$mid
frgrid=length(uifr)


### B-spline basis for the Frailty density specification
# ----------------------------------------------
bdegfr=3; pordfr=3; nsegfr=12; 			# degree of B-splines, order of penalty and number of knots
lBfr=bbase(uifr,-6.0,6.0,nsegfr,deg=bdegfr)
kfr=ncol(lBfr)


### Contruct the penalty matrix for the frailty
# ------------------------------------------
Dfr= diff(diag(kfr), diff = pordfr)				# Contruct the penalty matrix
Pfr=t(Dfr)%*%Dfr
Pfr=Pfr+1e-6*diag(kfr) 						# Remark: When we use full rank P matrix, RP=k
RPfr=kfr 								


### Plot of the Basis 
# ------------------------------------------
par(mfrow=c(1,1))
plot(uifr,lBfr[,1],type='l',ylim=c(0,1.),ylab="",xlab="")
for (j in 2:kfr) lines(uifr,lBfr[,j],col=j)


### Frailty density initilization using apoisson.r
# --------------------------------------------------
lambda = 10^seq(-4,3,by=.5)
bicfr = 0*lambda
for (j in 1:length(lambda)){
	ypoi=apoisson(x=uifr,y=pseu.ctfr,X=lBfr,nseg=nsegfr,bdeg=bdegfr,pord = pordfr,lambda = lambda[j], plot = F)
	bicfr[j] = ypoi$bic 
}

plot(log10(lambda),bicfr,type="b")
lambda0fr=lambda[bicfr==min(bicfr)] # Initial value of penalty parameter tau is selected in the grid


ypoifr=apoisson(x=uifr,y=pseu.ctfr,X=lBfr,nseg=nsegfr,bdeg=bdegfr,pord = pordfr,lambda = lambda0fr, plot = F)
thetamlfr=ypoifr$coef		# Spline coefficients
thetamlfr=standfr(thetamlfr)	# Identifiability constraint on the spline coefficients

SIGMAfr=ypoifr$covar		# variance covariance matrix for the spline coefficients for the frailty density 
t(chol(solve(SIGMAfr))) 
Lfr=t(chol(solve(t(lBfr)%*%lBfr+lambda0fr*Pfr))) 


eta0fr = lBfr%*%thetamlfr ; 
dufr = uifr[2]-uifr[1] ; 
f0fr = exp(eta0fr)/sum(exp(eta0fr))/dufr 		#Estimated frailty density  


### Moments of standardized frailty density 
# -----------------------------------------
meanfs=sum(uifr*f0fr*dufr)
varfs=sum((uifr-meanfs)^2*f0fr*dufr)
pen=1000*(meanfs^2+abs(varfs-1))


### FRAILTY: Visualisation of starting values 
# ------------------------------------------
x11()
plot(uifr,f0fr,lty=3,lwd=2,ylim=c(0,.4))
hist(fr.s,freq=F,breaks=seq(-6,6,length=30),add=T)


# THE BASIS  Characteristics
# ---------------------------------------------------------------------
bdeg=3; pord=2; nseg=13; # degree of B-splines, order of penalty and number of knots
du=0.1
ui=seq(du/2,tcens[1]-du/2,by=du)
lB=bbase(ui,0,tcens[1],nseg,deg=bdeg)
k=ncol(lB)


# Contruct the penalty matrix for baseline
# -----------------------------------------------
D= diff(diag(k), diff = pord)
P=t(D)%*%D
P=P+1e-6*diag(k) # Remark: When we use full rank P matrix, RP=k
RP=k 


### Baseline density of time to event  ##################
#########################################################

# Construct the small bins where du=0.1
# ---------------------------------------
sbin=seq(0,tcens[1],by=du) 


# since small bin width (du) is 0.1, we multiply by 1/du=10
# ------------------------------------------------------------
ylC=10*yl
yrC=10*yr
w.intC=yrC-ylC
tcensC=tcens/du
upC=up/du


# Specify the C matrix
# ---------------------------------------
Cm=matrix(rep(0,n*(length(sbin))),ncol=(length(sbin)),nrow=n)
CL=CR=matrix(rep(0,n*(length(sbin))),ncol=(length(sbin)),nrow=n)
Cmi=matrix(rep(0,n*(upC)),ncol=(upC),nrow=n)
Cmi.w=matrix(rep(0,n*(upC)),ncol=(upC),nrow=n)

for (i in 1:n) {
if (d[i]==1){
	#cat('i',i,'\n')

	CL[i,]=c(rep(0,ylC[i]+1e-7),rep(1,tcensC[i]-ylC[i]+1e-7),1)
	CR[i,]=c(rep(0,yrC[i]+1e-7),rep(1,length(sbin)-yrC[i]+1e-7))
	Cm[i,]=c(rep(0,ylC[i]+1e-7),rep(1,w.intC[i]+1e-7),rep(0,(tcensC[i]-yrC[i]+1e-7)),0)
	Cmi[i,]=c(rep(0,ylC[i]+1e-7),rep(1,w.intC[i]+1e-7),rep(0,(upC-yrC[i]+1e-7)))  } 
else {
	#cat('i d=0',i,"\n")
		CL[i,]=c(rep(0,ylC[i]+1e-7),rep(1,tcensC[i]-ylC[i]+1e-7),1)
		CR[i,]=c(rep(0,ylC[i]+1e-7),rep(0,tcensC[i]-ylC[i]+1e-7),0)
		Cm[i,]=c(rep(0,ylC[i]+1e-7),rep(1,(tcensC[i]-ylC[i]+1e-7)),1)
	      Cmi[i,]=c(rep(0,ylC[i]+1e-7),rep(1,(upC-ylC[i]+1e-7)))  
} 	
}
sumCm=apply(Cmi,1,sum)
for (i in 1:n){
if (sumCm[i]==0) {sumCm[i]=1} 
}

Cmi.w=Cmi/sumCm
Cmi.w=Cmi.w[,]
lm=tcens[1]/du+1
um=up/du
Cmi.w=Cmi.w[,-(lm:um)]


# Plot of the Basis 
# ---------------------------------------
par(mfrow=c(1,1))
plot(ui,lB[,1],type='l',yli=c(0,.7),ylab="",xlab="")
for (j in 1:k) lines(ui,lB[,j],col=j)


# Find the optimal lambda in the frequentist setting using POISSON approach
# --------------------------------------------------------------------
pseu.ct=apply(Cmi.w,2,sum);	ms=sum(pseu.ct)


# For initial estimates of MCMC use apoisson.r
# -------------------------------------------------
lambda = 10^seq(-2,4,by=.5)
aic = 0*lambda
for (j in 1:length(lambda)){
	ypoi=apoisson(x=ui,y=pseu.ct,X=lB,nseg=nseg,bdeg=bdeg,pord = pord,lambda = lambda[j], plot = F)
	aic[j] = ypoi$bic 	# Find lambda for which the smallest AIC is obtained.
	thetaml=ypoi$coef
	ygrid=as.vector(ypoi$ygrid)
}
plot(log10(lambda),aic,type="b")
lambda0=lambda[aic==min(aic)] # Optimal lambda in the grid
ypoi=apoisson(x=ui,y=pseu.ct,X=lB,nseg=nseg,bdeg=bdeg,pord = pord,lambda = lambda0, plot = F)
thetaml=ypoi$coef
thetaml=standbase(thetaml)


SIGMA=ypoi$covar
sdp=sqrt(diag(SIGMA+1e-6*diag(k)))
L=t(chol(solve(t(lB)%*%lB+P))) 	


# Visualisation of the starting values
# ---------------------------------------
x11()
eta0 = lB%*%thetaml ; 
du = ui[2]-ui[1] ; 
f0 = (ksi0/du)*exp(eta0)/sum(exp(eta0))
plot(ui,f0,lty="dashed",lwd=2,ylim=c(.0,.5))



### SECTION 2: Call Fortran code for flexible frailty model ######################
##################################################################################


# The arrays required to call Fortran functions
# --------------------------------------------------
M=153200			# Length of the chain
n.burnin=round(M*3/5) 	# Length of burn-in perior 
idx_save=seq(2,M,by=50)
Msave=length(idx_save) 	# Number of iterations that will be saved (due to autocorrelation)

t.end=tcens[1]/du
t.endn=t.end+1


# The arrays required to call Fortran
# ---------------------------------------
idx_save=seq(2,M,by=50)
Msave=length(idx_save)


logit.ksi=ksi=logit.ksip=numeric(Msave); 
tau=numeric(Msave); 
coeff=matrix(numeric(nZ*Msave),nZ,Msave);
taufr=numeric(Msave);
sigma=numeric(Msave);
fthet<-matrix(numeric(ncl*Msave),ncl,Msave)
phi<-matrix(numeric(k*Msave),k,Msave)
phifr<-matrix(numeric(kfr*Msave),kfr,Msave)
taufr=numeric(Msave)


delta=matrix(numeric(k*M),M,k); 		
deltaM=deltafrM=numeric(M);		
delta.ksi=numeric(M);				
delta.b=matrix(numeric(nZ*M),nZ,M)		
delta.tau=numeric(M)				
deltafthet=matrix(numeric(ncl*M),ncl,M)	
deltafthetf=matrix(numeric(M),M,1)	 


deltafr=matrix(numeric(kfr*M),M,kfr);	
delta.lsig=numeric(M);				

acc_coeff=rep(0,nZ)
acc_ksi=0
acc_sigb=0
acc_phi=rep(0,k)
acc_phifr=rep(0,kfr)
acc_fthets=rep(0,ncl)
acc.tau=acc.taufr=acc.phifrM=0
acc_phiM=0


### Setting initial values of the chains obtained from frequentist (naive) models
# --------------------------------------------------------------------------------
phi.init=rep(0,k)
phifr.init=rep(0,kfr)
coeff.init=coeff.fr 		
ltaufr.init=lambda0fr
lsig.init=lthet=log(sd(frterm))		
logit.ksi.init=log(ksi0/(1-ksi0))
taufr.init=lambda0fr
tau.init=lambda0
fthet.init=fr.s


lp=0

t.end=75 	# tcens[1]/du
t.endn=76 	# t.end+1




### CALL Fortran function 
# --------------------------------------------
setwd("C:/Users/AysunC/Documents/MCMC/Fortran")
dyn.load("logpostFRdeneme10MULTCoeffx.dll")
is.loaded("mhfr")
out=.Fortran("mhfr",MM=as.integer(M),idx_save=as.integer(idx_save),MMsave=as.integer(Msave),phix=as.double(phi.init),L=as.double(L),nk=as.integer(k),thetaml=as.double(thetaml),
lB=as.double(lB),tcens=as.integer(t.end),tcensn=as.integer(t.endn),
taufrx=as.double(taufr.init),logit_ksix=as.double(logit.ksi.init),lsigx=as.double(lsig.init),kfr=as.integer(kfr),Lfr=as.double(Lfr),phifrx=as.double(phifr.init),
thetamlfr=as.double(thetamlfr),lBfr=as.double(lBfr),frgrid=as.integer(frgrid),ncl=as.integer(ncl),fthetsx=as.double(fthet.init),
uifr=as.double(uifr),ni=as.integer(ni),n=as.integer(n),CL=as.double(CL),CR=as.double(CR),Z=as.double(Z),nZ=as.integer(nZ),coeffx=as.double(coeff.init)
,P=as.double(P),Pfr=as.double(Pfr),taux=as.double(tau.init),phif=as.double(phi),deltaf=as.double(delta),acc_phi=as.integer(acc_phi),tau_c=as.double(tau),
ksif=as.double(ksi),delta_ksi=as.double(delta.ksi),acc_ksi=as.integer(acc_ksi),coefff=as.double(coeff),delta_bf=as.double(delta.b),acc_coeff=as.integer(acc_coeff),
sigbf=as.double(sigma),delta_sigb=as.double(delta.lsig),acc_sigb=as.integer(acc_sigb),phifrf=as.double(phifr),acc_phifr=as.integer(acc_phifr),
deltafrf=as.double(deltafr),taufr_c=as.double(taufr),fthetsf=as.double(fthet),delta_fthetsf=as.double(deltafthet),acc_fthets=as.integer(acc_fthets),
lp=as.double(lp))
dyn.unload("logpostFRdeneme10MULTCoeffx.dll") 


# Acceptance probabilities
# --------------------------------------------
out$acc_phi/M
out$acc_phifr/M
out$acc_ksi/M
out$acc_coeff/M
out$acc_sigb/M


# Chains for model parameters
# --------------------------------------------
phif=matrix(unlist(out$phif),byrow=F,nrow=k,ncol=Msave)
phifrf=matrix(unlist(out$phifrf),byrow=F,nrow=kfr,ncol=Msave)
fthetf=matrix(unlist(out$fthetsf),byrow=F,nrow=ncl,ncol=Msave)
coefff=matrix(unlist(out$coefff),byrow=F,nrow=nZ,ncol=Msave)
ksif=matrix(unlist(out$ksif),byrow=T,nrow=1,ncol=Msave)
tauf=matrix(unlist(out$tau_c),byrow=T,nrow=1,ncol=Msave)
taufrf=matrix(unlist(out$taufr_c),byrow=T,nrow=1,ncol=Msave)
sigf=matrix(unlist(out$sigbf),byrow=T,nrow=1,ncol=Msave)


# Remove the burnin period
# --------------------------------------------
nburnsave=round(Msave*2/5)
phif=phif[,-(1:nburnsave) ]
phifrf=phifrf[,-(1:nburnsave) ]
coefff=coefff[,-(1:nburnsave) ]
tauf=tauf[-(1:nburnsave)]
taufrf=taufrf[-(1:nburnsave)]
sigf=sigf[-(1:nburnsave)]
ksif=ksif[-(1:nburnsave)]
fthetf=fthetf[,-(1:nburnsave) ]



# Final chains for the spline coefficients for baseline density
# --------------------------------
thetaf=t(thetaml+L%*%(phif))
theta=t(apply(thetaf,1,standbase))

# Final chains for the spline coefficients for frailty density
# --------------------------------
thetafrf=t(thetamlfr+Lfr%*%(phifrf))
thetafr=t(apply(thetafrf,1,standfr))


### SECTION 3: Presenting and visualising estimates from the model ######################
#############################################################################################
### traceplots of the model parameters


### traceplot of the estimated frailty terms
#----------------------------------------------
x11();
par(mfrow=c(3,3));
for (j in 2:10) {plot(fthetf[j,],type='l');}


### traceplot of the penalty parameters
#-----------------------------------------
x11();
par(mfrow=c(2,1))
plot(log10(tauf),type="l"); plot(log10(taufrf),type="l");


### traceplot of the spline coefficents for baseline density
#----------------------------------------------
x11();
par(mfrow=c(3,2));
for (j in 1:6) plot(theta[,j],type='l')

x11();
par(mfrow=c(3,2));
for (j in 7:12) plot(theta[,j],type='l')


### traceplot of ksi(=sum of the probability until end of observation period)
#----------------------------------------------------------------------------
x11();
plot(ksif,type='l')


### Traceplot of the regression coefficients
#------------------------------------------------------
x11();
par(mfrow=c(3,1));
plot(coefff[1,],type='l',lwd=2); 
plot(coefff[2,],type='l',lwd=2); 
plot(coefff[3,],type='l',lwd=2); 

x11();
par(mfrow=c(3,1));
plot(coefff[4,],type='l',lwd=2); 
plot(coefff[5,],type='l',lwd=2); 
plot(coefff[6,],type='l',lwd=2); 

x11();
par(mfrow=c(3,1));
plot(coefff[7,],type='l',lwd=2); 
plot(coefff[8,],type='l',lwd=2); 
plot(coefff[9,],type='l',lwd=2); 

x11();
par(mfrow=c(3,1));
plot(coefff[10,],type='l',lwd=2); 
plot(coefff[11,],type='l',lwd=2); 
plot(coefff[12,],type='l',lwd=2); 


x11();
par(mfrow=c(3,1));
plot(coefff[13,],type='l',lwd=2); 
plot(coefff[14,],type='l',lwd=2); 
plot(coefff[15,],type='l',lwd=2); 


### Traceplot of the spline coefficients for the frailty
#------------------------------------------------------
x11();
par(mfrow=c(3,2));
for (j in 1:6) plot(thetafrf[,j],type='l')

x11();
par(mfrow=c(3,2));
for (j in 7:12) plot(thetafrf[,j],type='l')


### Traceplot of the standard deviation of the frailty
#------------------------------------------------------
x11();
par(mfrow=c(1,2));
plot(sigf,type="l",ylim=c(0.5,3)); 
truehist(sigf,col='grey')


### Fitted FRAILTY density
# -----------------------------------------
eta.matfr = lBfr%*%t(thetafr); 
pi.matfrc=exp(eta.matfr)
dufr=uifr[2]-uifr[1]
#rescale.factorr = apply(pi.matfrc,2,sum)
pi.matfr<-matrix(numeric(nrow(pi.matfrc)*ncol(pi.matfrc)),nrow(pi.matfrc),ncol(pi.matfrc))

for (i in 1:ncol(pi.matfr)) pi.matfr[,i]=pi.matfrc[,i]/dufr


f.meanfr = apply(pi.matfr,1,median)
f.05mfr = apply(pi.matfr,1,quantile,p=.05)
f.95mfr = apply(pi.matfr,1,quantile,p=.95)


### Moments of standardized frailty density 
#-----------------------------------------------
meanfr=sum(uifr*f.meanfr*dufr)
varfr=sum((uifr-meanfr)^2*f.meanfr*dufr)


### Plot of the estimated frailty density with 90% credible intervals 
# ---------------------------------------------------------------------
x11();
hist(fr.s,col="lightgrey",ylim=c(0,.55),breaks=seq(-6,6,length=25),freq=F,main="Estimated frailty density")
lines(uifr,f.meanfr,lwd=3,type='l',ylim=c(0,.75),xlab="",xlim=c(-6,6),col='red') # Fitted density
lines(uifr,f.05mfr,lty="dashed",lwd=2.5,col='black') # Pointwise credible interval
lines(uifr,f.95mfr,lty="dashed",lwd=2.5,col='black') # 
legend(2.,0.43,bty="n",cex=0.85,lwd=c(3,2.5,3),lty=c(1,2,3),col=c(2,1,4),legend=c("Median","90% Cred. Int."))


x11();
plot(uifr,f.meanfr,lwd=2,type='l',ylim=c(0,.7),ylab="probability",xlab="frailty",xlim=c(-6,6),col='black',main="Estimated frailty density") # Fitted density
polygon(c(uifr,uifr),c(f.95mfr,f.95mfr),col='lightgray',lwd=2,border=NULL,density=c(85))
polygon(c(uifr,uifr),c(f.05mfr,f.05mfr),col='white',lwd=2,border=NULL,density=c(85))
lines(uifr,f.meanfr,lwd=2,type='l',ylim=c(0,.75),xlab="",xlim=c(-6,6),col='black') # Fitted density



### Estimated baseline density on a grid
# ----------------------------------------------
eta.mat = lB%*%t(theta); 
pi.matc=exp(eta.mat)
du=ui[2]-ui[1]
pi.mat<-matrix(numeric(nrow(pi.matc)*ncol(pi.matc)),nrow(pi.matc),ncol(pi.matc))

for (i in 1:ncol(pi.matc)) pi.mat[,i]=ksif[i]*pi.matc[,i]/du 


### traceplot of the estimated density on a grid
# ----------------------------------
x11();
par(mfrow=c(3,3));
for (j in 60:68) plot(pi.mat[j,],type='l')


### Average of the estimated baseline density and 90% credible intervals
#-----------------------------------------------------------------------
f.mean = apply(pi.mat,1,mean)
f.05m = apply(pi.mat,1,quantile,p=.05)
f.95m = apply(pi.mat,1,quantile,p=.95)


# Plotting the fitted baseline and the target baseline density 
# --------------------------------------------------------------------
x11();
plot(ui+5,f.mean,lwd=2,type='l',col='red',ylim=c(0,0.9),xlim=c(5,12),xlab="Time",ylab="Emergence",main="Estimated baseline density of emergence times") # Fitted density
lines(ui+5,f.05m,lty="dashed",lwd=2) 	# Pointwise credible interval
lines(ui+5,f.95m,lty="dashed",lwd=2) 	
abline(v=tcens[1]+5,col='purple',lwd=2.5,lty=4)


# Estimate survival function and cumulative distribution function at the midpoints
# ---------------------------------------------------------------------------- 
cumpi.mat=sf.mat=matrix(rep(0,tcensC[1]*(Msave-nburnsave)),nrow=tcensC[1])
cumf=apply(pi.mat*du,2,cumsum)
cumpi.mat[1,]=(pi.mat[1,]*du/2)
sf.mat[1,]=1-cumpi.mat[1,]
for (i in 2:nrow(pi.mat)){
 	cumpi.mat[i,]=(cumf[i-1,]+pi.mat[i,]*du/2)
 	sf.mat[i,]=1-cumpi.mat[i,]
}

cumf.mean = apply(cumpi.mat,1,median)
cumf.05m = apply(cumpi.mat,1,quantile,p=.05)
cumf.95m = apply(cumpi.mat,1,quantile,p=.95)

cumf.mean[cumf.mean>1]=1.0
cumf.05m[cumf.05m>1]=1.0
cumf.95m[cumf.95m>1]=1.0


### Survival curves for the levels of given covariates are estimated (Figure 3 in main paper)
# ------------------------------------------------------------------------------------------------
sftr1=sftr2=sftr21=sftr22=sftr3=sftr32=sftr4=sftr42=sftr5=sftr52=sftr6=sftr62=sftr72=sftr7=sftr9=sftr92=matrix(numeric(length(ui)*(Msave-nburnsave)),nrow=length(ui))
for (i in 1:length(ui)) { 
	sftr1[i,]=(sf.mat[i,])^exp(coefff[1,])
	sftr2[i,]=(sf.mat[i,])^exp(coefff[2,])
	sftr21[i,]=(sf.mat[i,])^exp(coefff[1,]+coefff[2,]+coefff[3,])
	sftr3[i,]=(sf.mat[i,])^exp(coefff[2,]+coefff[4,]+coefff[7,])
	sftr32[i,]=(sf.mat[i,])^exp(coefff[1,]+coefff[2,]+coefff[3,]+coefff[4,]+coefff[7,]+coefff[10,]+coefff[13,])
	sftr4[i,]=(sf.mat[i,])^exp(coefff[4,])
	sftr42[i,]=(sf.mat[i,])^exp(coefff[1,]+coefff[4,]+coefff[10,])
	sftr22[i,]=(sf.mat[i,])^exp(coefff[1,]+coefff[3,])
	sftr5[i,]=(sf.mat[i,])^exp(coefff[2,]+coefff[5,]+coefff[8,])
	sftr52[i,]=(sf.mat[i,])^exp(coefff[1,]+coefff[2,]+coefff[3,]+coefff[5,]+coefff[8,]+coefff[11,]+coefff[14,])
	sftr6[i,]=(sf.mat[i,])^exp(coefff[5,])
	sftr62[i,]=(sf.mat[i,])^exp(coefff[1,]+coefff[5,]+coefff[11,])
	sftr7[i,]=(sf.mat[i,])^exp(coefff[2,]+coefff[6,]+coefff[9,])
	sftr72[i,]=(sf.mat[i,])^exp(coefff[1,]+coefff[2,]+coefff[3,]+coefff[6,]+coefff[9,]+coefff[12,]+coefff[15,])
	sftr9[i,]=(sf.mat[i,])^exp(coefff[6,])
	sftr92[i,]=(sf.mat[i,])^exp(coefff[1,]+coefff[6,]+coefff[12,])
}


sf.mat[sf.mat<0]=0

sf.mean = apply(sf.mat,1,median)
sf.05m= apply(sf.mat,1,quantile,p=.05)
sf.95m= apply(sf.mat,1,quantile,p=.95)

sf.mean[sf.mean>1]=0.0
sf.05m[sf.05m>1]=0.0
sf.95m[sf.95m>1]=0.0

sftr1[is.na(sftr1)]=0

sftr1.mean=apply(sftr1,1,mean)
sftr1.05m=apply(sftr1,1,quantile,p=.05)
sftr1.95m=apply(sftr1,1,quantile,p=.95)

sftr1.mean[sftr1.mean>1]=1.0
sftr1.05m[sftr1.05m>1]=0.0
sftr1.95m[sftr1.95m>1]=0.0

sftr2.mean=apply(sftr2,1,mean)
sftr2.05m=apply(sftr2,1,quantile,p=.05)
sftr2.95m=apply(sftr2,1,quantile,p=.95)

sftr2.mean[sftr2.mean>1]=1.0
sftr2.05m[sftr2.05m>1]=0.0
sftr2.95m[sftr2.95m>1]=0.0

sftr21.mean=apply(sftr21,1,mean)
sftr21.05m=apply(sftr21,1,quantile,p=.05)
sftr21.95m=apply(sftr21,1,quantile,p=.95)

sftr21.mean[sftr21.mean>1]=1.0
sftr21.05m[sftr21.05m>1]=0.0
sftr21.95m[sftr21.95m>1]=0.0

sftr22.mean=apply(sftr22,1,mean)
sftr22.05m=apply(sftr22,1,quantile,p=.05)
sftr22.95m=apply(sftr22,1,quantile,p=.95)

sftr22.mean[sftr22.mean>1]=1.0
sftr22.05m[sftr22.05m>1]=0.0
sftr22.95m[sftr22.95m>1]=0.0


sftr3.mean=apply(sftr3,1,mean)
sftr3.05m=apply(sftr3,1,quantile,p=.05)
sftr3.95m=apply(sftr3,1,quantile,p=.95)

sftr3.mean[sftr3.mean>1]=1.0
sftr3.05m[sftr3.05m>1]=0.0
sftr3.95m[sftr3.95m>1]=0.0


sftr32.mean=apply(sftr32,1,mean)
sftr32.05m=apply(sftr32,1,quantile,p=.05)
sftr32.95m=apply(sftr32,1,quantile,p=.95)

sftr32.mean[sftr32.mean>1]=1.0
sftr32.05m[sftr32.05m>1]=0.0
sftr32.95m[sftr32.95m>1]=0.0

sftr4.mean=apply(sftr4,1,mean)
sftr4.05m=apply(sftr4,1,quantile,p=.05)
sftr4.95m=apply(sftr4,1,quantile,p=.95)

sftr4.mean[sftr4.mean>1]=1.0
sftr4.05m[sftr4.05m>1]=0.0
sftr4.95m[sftr4.95m>1]=0.0

sftr42.mean=apply(sftr42,1,mean)
sftr42.05m=apply(sftr42,1,quantile,p=.05)
sftr42.95m=apply(sftr42,1,quantile,p=.95)

sftr42.mean[sftr42.mean>1]=1.0
sftr42.05m[sftr42.05m>1]=0.0
sftr42.95m[sftr42.95m>1]=0.0


sftr5.mean=apply(sftr5,1,mean)
sftr5.05m=apply(sftr5,1,quantile,p=.05)
sftr5.95m=apply(sftr5,1,quantile,p=.95)

sftr5.mean[sftr5.mean>1]=1.0
sftr5.05m[sftr5.05m>1]=0.0
sftr5.95m[sftr5.95m>1]=0.0


sftr52.mean=apply(sftr52,1,mean)
sftr52.05m=apply(sftr52,1,quantile,p=.05)
sftr52.95m=apply(sftr52,1,quantile,p=.95)

sftr52.mean[sftr52.mean>1]=1.0
sftr52.05m[sftr52.05m>1]=0.0
sftr52.95m[sftr52.95m>1]=0.0

sftr6.mean=apply(sftr6,1,mean)
sftr6.05m=apply(sftr6,1,quantile,p=.05)
sftr6.95m=apply(sftr6,1,quantile,p=.95)

sftr6.mean[sftr6.mean>1]=1.0
sftr6.05m[sftr6.05m>1]=0.0
sftr6.95m[sftr6.95m>1]=0.0

sftr62.mean=apply(sftr62,1,mean)
sftr62.05m=apply(sftr62,1,quantile,p=.05)
sftr62.95m=apply(sftr62,1,quantile,p=.95)

sftr62.mean[sftr62.mean>1]=1.0
sftr62.05m[sftr62.05m>1]=0.0
sftr62.95m[sftr62.95m>1]=0.0

sftr7.mean=apply(sftr7,1,mean)
sftr7.05m=apply(sftr7,1,quantile,p=.05)
sftr7.95m=apply(sftr7,1,quantile,p=.95)

sftr7.mean[sftr7.mean>1]=1.0
sftr7.05m[sftr7.05m>1]=0.0
sftr7.95m[sftr7.95m>1]=0.0

sftr72.mean=apply(sftr72,1,mean)
sftr72.05m=apply(sftr72,1,quantile,p=.05)
sftr72.95m=apply(sftr72,1,quantile,p=.95)

sftr72.mean[sftr72.mean>1]=1.0
sftr72.05m[sftr72.05m>1]=0.0
sftr72.95m[sftr72.95m>1]=0.0

sftr9.mean=apply(sftr9,1,mean)
sftr9.05m=apply(sftr9,1,quantile,p=.05)
sftr9.95m=apply(sftr9,1,quantile,p=.95)

sftr9.mean[sftr9.mean>1]=1.0
sftr9.05m[sftr9.05m>1]=0.0
sftr9.95m[sftr9.95m>1]=0.0

sftr92.mean=apply(sftr92,1,mean)
sftr92.05m=apply(sftr92,1,quantile,p=.05)
sftr92.95m=apply(sftr92,1,quantile,p=.95)

sftr92.mean[sftr92.mean>1]=1.0
sftr92.05m[sftr92.05m>1]=0.0
sftr92.95m[sftr92.95m>1]=0.0

uis=sort(ui+5,decreasing=TRUE)
sf.95ms=sort(sf.95m,decreasing=FALSE)
sftr1.95ms=sort(sftr1.95m,decreasing=FALSE)
sftr2.95ms=sort(sftr2.95m,decreasing=FALSE)
sftr21.95ms=sort(sftr21.95m,decreasing=FALSE)
sftr22.95ms=sort(sftr22.95m,decreasing=FALSE)
sftr3.95ms=sort(sftr3.95m,decreasing=FALSE)
sftr32.95ms=sort(sftr32.95m,decreasing=FALSE)
sftr4.95ms=sort(sftr4.95m,decreasing=FALSE)
sftr42.95ms=sort(sftr42.95m,decreasing=FALSE)
sftr5.95ms=sort(sftr5.95m,decreasing=FALSE)
sftr52.95ms=sort(sftr52.95m,decreasing=FALSE)
sftr6.95ms=sort(sftr6.95m,decreasing=FALSE)
sftr62.95ms=sort(sftr62.95m,decreasing=FALSE)
sftr72.95ms=sort(sftr72.95m,decreasing=FALSE)
sftr7.95ms=sort(sftr7.95m,decreasing=FALSE)
sftr9.95ms=sort(sftr9.95m,decreasing=FALSE)
sftr92.95ms=sort(sftr92.95m,decreasing=FALSE)



# Max 4 Girls
x11();
plot(ui+5,sftr2.mean,lwd=2,type='l',xlim=c(0+5,12),ylim=c(0,1),xlab="Age (years)",ylab="survival",main="Maxilla 4, Girls") # Fitted density
lines(ui+5,sftr21.mean,lty=4,lwd=2)
polygon(c(ui+5,uis),c(sftr2.05m,sftr2.95ms),col='lightgray',lwd=1,border=NULL,density=c(15))
polygon(c(ui+5,uis),c(sftr21.05m,sftr21.95ms),col='lightgray',lwd=1,border=NULL,density=c(5))
legend(6,0.53,bty="n",cex=1.05,lwd=c(2,2),lty=c(1,4),legend=c("dmf=0","dmf=1"))


# Max 4 Boys
x11();
plot(ui+5,sf.mean,lwd=2,type='l',xlim=c(0+5,12),ylim=c(0,1),xlab="Age (years)",ylab="survival",main="Maxilla 4, Boys") # Fitted density
lines(ui+5,sftr1.mean,lty=4,lwd=2)
polygon(c(ui+5,uis),c(sf.05m,sf.95ms),col='lightgray',lwd=1,border=NULL,density=c(15))
polygon(c(ui+5,uis),c(sftr1.05m,sftr1.95ms),col='lightgray',lwd=1,border=NULL,density=c(5))
legend(6,0.53,bty="n",cex=1.05,lwd=c(2,2),lty=c(1,4),legend=c("dmf=0","dmf=1"))


# Man 4 Girls
x11();
plot(ui+5,sftr3.mean,lwd=2,type='l',xlim=c(0+5,12),ylim=c(0,1),xlab="Age (years)",ylab="survival",main="Mandible 4, Girls") # Fitted density
lines(ui+5,sftr32.mean,lty=4,lwd=2)
polygon(c(ui+5,uis),c(sftr3.05m,sftr3.95ms),col='lightgray',lwd=1,border=NULL,density=c(15))
polygon(c(ui+5,uis),c(sftr32.05m,sftr32.95ms),col='lightgray',lwd=1,border=NULL,density=c(5))
legend(6,0.53,bty="n",cex=1.05,lwd=c(2,2),lty=c(1,4),legend=c("dmf=0","dmf=1"))


# Man 4 Boys
x11();
plot(ui+5,sftr4.mean,lwd=2,type='l',xlim=c(0+5,12),ylim=c(0,1),xlab="Age (years)",ylab="survival",main="Mandible 4, Boys") # Fitted density
lines(ui+5,sftr42.mean,lty=4,lwd=2)
polygon(c(ui+5,uis),c(sftr4.05m,sftr4.95ms),col='lightgray',lwd=1,border=NULL,density=c(15))
polygon(c(ui+5,uis),c(sftr42.05m,sftr42.95ms),col='lightgray',lwd=1,border=NULL,density=c(5))
legend(6,0.53,bty="n",cex=1.05,lwd=c(2,2),lty=c(1,4),legend=c("dmf=0","dmf=1"))
lines(ui+5,sftr4.mean,lwd=1,) # Fitted density


# Max 5 Girls
x11();
plot(ui+5,sftr5.mean,lwd=2,type='l',xlim=c(0+5,12),ylim=c(0,1),xlab="Age (years)",ylab="survival",main="Maxilla 5, Girls") # Fitted density
lines(ui+5,sftr52.mean,lty=4,lwd=2)
polygon(c(ui+5,uis),c(sftr5.05m,sftr5.95ms),col='lightgray',lwd=1,border=NULL,density=c(15))
polygon(c(ui+5,uis),c(sftr52.05m,sftr52.95ms),col='lightgray',lwd=1,border=NULL,density=c(5))
legend(6,0.53,bty="n",cex=1.05,lwd=c(2,2),lty=c(1,4),legend=c("dmf=0","dmf=1"))

# Max 5 Boys
x11();
plot(ui+5,sftr6.mean,lwd=2,type='l',xlim=c(0+5,12),ylim=c(0,1),xlab="Age (years)",ylab="survival",main="Maxilla 5, Boys") # Fitted density
lines(ui+5,sftr62.mean,lty=4,lwd=2)
polygon(c(ui+5,uis),c(sftr6.05m,sftr6.95ms),col='lightgray',lwd=1,border=NULL,density=c(15))
polygon(c(ui+5,uis),c(sftr62.05m,sftr62.95ms),col='lightgray',lwd=1,border=NULL,density=c(5))
legend(6,0.53,bty="n",cex=1.05,lwd=c(2,2),lty=c(1,4),legend=c("dmf=0","dmf=1"))



# Man 5 Girls
x11();
plot(ui+5,sftr7.mean,lwd=2,type='l',xlim=c(0+5,12),ylim=c(0,1),xlab="Age (years)",ylab="survival",main="Mandible 5, Girls") # Fitted density
lines(ui+5,sftr72.mean,lty=4,lwd=2)
polygon(c(ui+5,uis),c(sftr7.05m,sftr7.95ms),col='lightgray',lwd=1,border=NULL,density=c(15))
polygon(c(ui+5,uis),c(sftr72.05m,sftr72.95ms),col='lightgray',lwd=1,border=NULL,density=c(5))
legend(6,0.53,bty="n",cex=1.05,lwd=c(2,2),lty=c(1,4),legend=c("dmf=0","dmf=1"))


# Man 5 Boys
x11();
plot(ui+5,sftr9.mean,lwd=2,type='l',xlim=c(0+5,12),ylim=c(0,1),xlab="Age (years)",ylab="survival",main="Mandible 5, Boys") # Fitted density
lines(ui+5,sftr92.mean,lty=4,lwd=2)
polygon(c(ui+5,uis),c(sftr9.05m,sftr9.95ms),col='lightgray',lwd=1,border=NULL,density=c(15))
polygon(c(ui+5,uis),c(sftr92.05m,sftr92.95ms),col='lightgray',lwd=1,border=NULL,density=c(5))
legend(6,0.53,bty="n",cex=1.05,lwd=c(2,2),lty=c(1,4),legend=c("dmf=0","dmf=1"))


### Table 1 in the supplement 
### MEdian and 90% credible interval for the estimates for the regression coefficients of the covariates
#----------------------------------------------------------------------------------------------
round(t(apply(coefff,1,quantile,p=c(0.05,.5,.95))),3)


### Median and 90%Credible Interval for standard deviation of the frailty 
#----------------------------------------------------------------------------------------------
quantile(sigf,p=c(0.05,.5,.95))




